@@ -19,33 +19,32 @@
 #include <Rcpp.h>
 #include <gdtools.h>
 #include <string>
+#include <iomanip>
+#include <boost/shared_ptr.hpp>
 #include <R_ext/GraphicsEngine.h>
 
+#include "SvgStream.h"
+
+typedef boost::shared_ptr<SvgStream> SvgStreamPtr;
+
 // SVG device metadata
 class SVGDesc {
 public:
-  FILE *file;
-  std::string filename;
+  SvgStreamPtr stream;
+
   int pageno;
-  double clipleft, clipright, cliptop, clipbottom;
+  int clipno;  // ID for the clip path
+  double clipx0, clipx1, clipy0, clipy1;  // Save the previous clip path to avoid duplication
   bool standalone;
   XPtrCairoContext cc;
 
-  SVGDesc(std::string filename_, bool standalone_):
-      filename(filename_),
+  SVGDesc(SvgStreamPtr stream_, bool standalone_):
+      stream(stream_),
       pageno(0),
+      clipno(0),
+      clipx0(0), clipx1(0), clipy0(0), clipy1(0),
       standalone(standalone_),
       cc(gdtools::context_create()) {
-    file = fopen(R_ExpandFileName(filename.c_str()), "w");
-  }
-
-  bool ok() const {
-    return file != NULL;
-  }
-
-  ~SVGDesc() {
-    if (ok())
-      fclose(file);
   }
 };
 
@@ -80,79 +79,86 @@ inline std::string fontname(const char* family_, int face) {
   }
 }
 
-inline void write_escaped(FILE* f, const char* text) {
+inline void write_escaped(SvgStreamPtr stream, const char* text) {
   for(const char* cur = text; *cur != '\0'; ++cur) {
     switch(*cur) {
-    case '&': fputs("&amp;", f); break;
-    case '<': fputs("&lt;",  f); break;
-    case '>': fputs("&gt;",  f); break;
-    default:  fputc(*cur,    f);
+    case '&': (*stream) << "&amp;"; break;
+    case '<': (*stream) << "&lt;";  break;
+    case '>': (*stream) << "&gt;";  break;
+    default:  (*stream) << *cur;
     }
   }
 }
 
-inline void write_attr_dbl(FILE* f, const char* attr, double value) {
-  fprintf(f, " %s='%.2f'", attr, value);
+inline void write_attr_dbl(SvgStreamPtr stream, const char* attr, double value) {
+  (*stream) << ' ' << attr << "='" << value << '\'';
 }
 
-inline void write_attr_str(FILE* f, const char* attr, const char* value) {
-  fprintf(f, " %s='%s'", attr, value);
+inline void write_attr_str(SvgStreamPtr stream, const char* attr, const char* value) {
+  (*stream) << ' ' << attr << "='" << value << '\'';
 }
 
+// Writing clip path attribute
+inline void write_attr_clip(SvgStreamPtr stream, int clipno) {
+  if (clipno < 1)
+    return;
 
+  (*stream) << " clip-path='url(#cp" << clipno << ")'";
+}
 
 // Beginning of writing style attributes
-inline void write_style_begin(FILE* f) {
-  fputs(" style='", f);
+inline void write_style_begin(SvgStreamPtr stream) {
+  (*stream) << " style='";
 }
 
 // End of writing style attributes
-inline void write_style_end(FILE* f) {
-  fputs("'", f);
+inline void write_style_end(SvgStreamPtr stream) {
+  (*stream) << "'";
 }
 
 // Writing style attributes related to colors
-inline void write_style_col(FILE* f, const char* attr, int col, bool first = false) {
+inline void write_style_col(SvgStreamPtr stream, const char* attr, int col, bool first = false) {
   int alpha = R_ALPHA(col);
 
-  fprintf(f, "%s", first ? "" : " ");
+  if(!first)  (*stream) << ' ';
+
   if (col == NA_INTEGER || alpha == 0) {
-    fprintf(f, "%s: none;", attr);
+    (*stream) << attr << ": none;";
     return;
   } else {
-    fprintf(f, "%s: #%02X%02X%02X;", attr, R_RED(col), R_GREEN(col), R_BLUE(col));
+    (*stream) << tfm::format("%s: #%02X%02X%02X;", attr, R_RED(col), R_GREEN(col), R_BLUE(col));
     if (alpha != 255)
-      fprintf(f, " %s-opacity: %0.2f;", attr, alpha / 255.0);
+      (*stream) << ' ' << attr << "-opacity: " << alpha / 255.0 << ';';
   }
 }
 
 // Writing style attributes whose values are double type
-inline void write_style_dbl(FILE* f, const char* attr, double value, bool first = false) {
-  fprintf(f, "%s", first ? "" : " ");
-  fprintf(f, "%s: %.2f;", attr, value);
+inline void write_style_dbl(SvgStreamPtr stream, const char* attr, double value, bool first = false) {
+  if(!first)  (*stream) << ' ';
+  (*stream) << attr << ": " << value << ';';
 }
 
-inline void write_style_fontsize(FILE* f, double value, bool first = false) {
-  fprintf(f, "%s", first ? "" : " ");
-  fprintf(f, "font-size: %.2fpt;", value);
+inline void write_style_fontsize(SvgStreamPtr stream, double value, bool first = false) {
+  if(!first)  (*stream) << ' ';
+  (*stream) << "font-size: " << value << "pt;";
 }
 
 // Writing style attributes whose values are strings
-inline void write_style_str(FILE* f, const char* attr, const char* value, bool first = false) {
-  fprintf(f, "%s", first ? "" : " ");
-  fprintf(f, "%s: %s;", attr, value);
+inline void write_style_str(SvgStreamPtr stream, const char* attr, const char* value, bool first = false) {
+  if(!first)  (*stream) << ' ';
+  (*stream) << attr << ": " << value << ';';
 }
 
 // Writing style attributes related to line types
-inline void write_style_linetype(FILE* f, const pGEcontext gc, bool first = false) {
+inline void write_style_linetype(SvgStreamPtr stream, const pGEcontext gc, bool first = false) {
   int lty = gc->lty;
 
   // 1 lwd = 1/96", but units in rest of document are 1/72"
-  write_style_dbl(f, "stroke-width", gc->lwd / 96 * 72, first);
+  write_style_dbl(stream, "stroke-width", gc->lwd / 96.0 * 72, first);
 
   // Default is "stroke: #000000;" as declared in <style>
   if (!is_black(gc->col))
-    write_style_col(f, "stroke", gc->col);
+    write_style_col(stream, "stroke", gc->col);
 
   // Set line pattern type
   switch (lty) {
@@ -161,16 +167,16 @@ inline void write_style_linetype(FILE* f, const pGEcontext gc, bool first = fals
     break;
   default:
     // See comment in GraphicsEngine.h for how this works
-    fputs(" stroke-dasharray: ", f);
+    (*stream) << " stroke-dasharray: ";
     // First number
-    fprintf(f, "%i", (int) gc->lwd * (lty & 15));
+    (*stream) << (int) gc->lwd * (lty & 15);
     lty = lty >> 4;
     // Remaining numbers
     for(int i = 1 ; i < 8 && lty & 15; i++) {
-      fprintf(f, ",%i", (int) gc->lwd * (lty & 15));
+      (*stream) << ',' << (int) gc->lwd * (lty & 15);
       lty = lty >> 4;
     }
-    fputs(";", f);
+    stream->put(';');
     break;
   }
 
@@ -180,10 +186,10 @@ inline void write_style_linetype(FILE* f, const pGEcontext gc, bool first = fals
   case GE_ROUND_CAP: // declared to be default in <style>
     break;
   case GE_BUTT_CAP:
-    write_style_str(f, "stroke-linecap", "butt");
+    write_style_str(stream, "stroke-linecap", "butt");
     break;
   case GE_SQUARE_CAP:
-    write_style_str(f, "stroke-linecap", "square");
+    write_style_str(stream, "stroke-linecap", "square");
     break;
   default:
     break;
@@ -195,18 +201,19 @@ inline void write_style_linetype(FILE* f, const pGEcontext gc, bool first = fals
   case GE_ROUND_JOIN: // declared to be default in <style>
     break;
   case GE_BEVEL_JOIN:
-    write_style_str(f, "stroke-linejoin", "bevel");
+    write_style_str(stream, "stroke-linejoin", "bevel");
     break;
   case GE_MITRE_JOIN:
-    write_style_str(f, "stroke-linejoin", "miter");
+    write_style_str(stream, "stroke-linejoin", "miter");
     if (std::abs(gc->lmitre - 10.0) > 1e-3) // 10 is declared to be the default in <style>
-      write_style_dbl(f, "stroke-miterlimit", gc->lmitre);
+      write_style_dbl(stream, "stroke-miterlimit", gc->lmitre);
     break;
   default:
     break;
   }
 }
 
+
 // Callback functions for graphics device --------------------------------------
 
 void svg_metric_info(int c, const pGEcontext gc, double* ascent,
@@ -226,106 +233,142 @@ void svg_metric_info(int c, const pGEcontext gc, double* ascent,
     gc->cex * gc->ps, is_bold(gc->fontface), is_italic(gc->fontface));
   FontMetric fm = gdtools::context_extents(svgd->cc, std::string(str));
 
-  *ascent = fm.ascent;
-  *descent = fm.descent;
-  *width = fm.width;
+  *ascent = fm.ascent * 96.0 / 72;
+  *descent = fm.descent * 96.0 / 72;
+  *width = fm.width * 96.0 / 72;
 }
 
 void svg_clip(double x0, double x1, double y0, double y1, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
-  svgd->clipleft = x0;
-  svgd->clipright = x1;
-  svgd->clipbottom = y0;
-  svgd->cliptop = y1;
+  // Avoid duplication
+  if (std::abs(x0 - svgd->clipx0) < 0.01 &&
+      std::abs(x1 - svgd->clipx1) < 0.01 &&
+      std::abs(y0 - svgd->clipy0) < 0.01 &&
+      std::abs(y1 - svgd->clipy1) < 0.01)
+    return;
+
+  svgd->clipno++;
+  svgd->clipx0 = x0;
+  svgd->clipx1 = x1;
+  svgd->clipy0 = y0;
+  svgd->clipy1 = y1;
+
+  (*stream) << "<defs>\n";
+  (*stream) << "  <clipPath id='cp" << svgd->clipno << "'>\n";
+  (*stream) << "    <rect x='" << std::min(x0, x1) << "' y='" << std::min(y0, y1) <<
+    "' width='" << std::abs(x1 - x0) << "' height='" << std::abs(y1 - y0) << "' />\n";
+  (*stream) << "  </clipPath>\n";
+  (*stream) << "</defs>\n";
 }
 
 void svg_new_page(const pGEcontext gc, pDevDesc dd) {
+BEGIN_RCPP
+
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
   if (svgd->pageno > 0) {
-    Rf_error("svglite only supports one page");
+    Rcpp::stop("svglite only supports one page");
   }
 
   if (svgd->standalone)
-    fputs("<?xml version='1.0' encoding='UTF-8' ?>\n", svgd->file);
+    (*stream) << "<?xml version='1.0' encoding='UTF-8' ?>\n";
 
-  fputs("<svg", svgd->file);
+  (*stream) << "<svg";
   if (svgd->standalone){
-    fputs(" xmlns='http://www.w3.org/2000/svg'", svgd->file);
+    (*stream) << " xmlns='http://www.w3.org/2000/svg'";
     //http://www.w3.org/wiki/SVG_Links
-    fputs(" xmlns:xlink='http://www.w3.org/1999/xlink'", svgd->file);
+    (*stream) << " xmlns:xlink='http://www.w3.org/1999/xlink'";
   }
 
-  fprintf(svgd->file, " viewBox='0 0 %.2f %.2f'>\n", dd->right, dd->bottom);
+  (*stream) << " viewBox='0 0 " << dd->right << ' ' << dd->bottom << "'>\n";
+
+  // Initialise clipping the same way R does
+  svgd->clipx0 = 0;
+  svgd->clipy0 = dd->bottom;
+  svgd->clipx1 = dd->right;
+  svgd->clipy1 = 0;
 
   // Setting default styles
-  fputs("<defs>\n", svgd->file);
-  fputs("  <style type='text/css'><![CDATA[\n", svgd->file);
-  fputs("    line, polyline, path, rect, circle {\n", svgd->file);
-  fputs("      fill: none;\n", svgd->file);
-  fputs("      stroke: #000000;\n", svgd->file);
-  fputs("      stroke-linecap: round;\n", svgd->file);
-  fputs("      stroke-linejoin: round;\n", svgd->file);
-  fputs("      stroke-miterlimit: 10.00;\n", svgd->file);
-  fputs("    }\n", svgd->file);
-  fputs("  ]]></style>\n", svgd->file);
-  fputs("</defs>\n", svgd->file);
+  (*stream) << "<defs>\n";
+  (*stream) << "  <style type='text/css'><![CDATA[\n";
+  (*stream) << "    line, polyline, path, rect, circle {\n";
+  (*stream) << "      fill: none;\n";
+  (*stream) << "      stroke: #000000;\n";
+  (*stream) << "      stroke-linecap: round;\n";
+  (*stream) << "      stroke-linejoin: round;\n";
+  (*stream) << "      stroke-miterlimit: 10.00;\n";
+  (*stream) << "    }\n";
+  (*stream) << "  ]]></style>\n";
+  (*stream) << "</defs>\n";
 
-  fputs("<rect width='100%' height='100%'", svgd->file);
-  write_style_begin(svgd->file);
-  write_style_str(svgd->file, "stroke", "none", true);
+  (*stream) << "<rect width='100%' height='100%'";
+  write_style_begin(stream);
+  write_style_str(stream, "stroke", "none", true);
   if (is_filled(gc->fill))
-    write_style_col(svgd->file, "fill", gc->fill);
+    write_style_col(stream, "fill", gc->fill);
   else
-    write_style_col(svgd->file, "fill", dd->startfill);
-  write_style_end(svgd->file);
-  fputs("/>\n", svgd->file);
+    write_style_col(stream, "fill", dd->startfill);
+  write_style_end(stream);
+  (*stream) << "/>\n";
 
+  svgd->stream->flush();
   svgd->pageno++;
+
+VOID_END_RCPP
 }
 
 void svg_close(pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
 
-  if (svgd->pageno > 0)
-    fputs("</svg>\n", svgd->file);
-
+  *(svgd->stream) << "</svg>\n";
   delete(svgd);
 }
 
 void svg_line(double x1, double y1, double x2, double y2,
                      const pGEcontext gc, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
-  fprintf(svgd->file, "<line x1='%.2f' y1='%.2f' x2='%.2f' y2='%.2f'",
-    x1, y1, x2, y2);
+  (*stream) << "<line x1='" << x1 << "' y1='" << y1 << "' x2='" <<
+    x2 << "' y2='" << y2 << '\'';
 
-  write_style_begin(svgd->file);
-  write_style_linetype(svgd->file, gc, true);
-  write_style_end(svgd->file);
+  write_style_begin(stream);
+  write_style_linetype(stream, gc, true);
+  write_style_end(stream);
 
-  fputs(" />\n", svgd->file);
+  write_attr_clip(stream, svgd->clipno);
+
+  (*stream) << " />\n";
+  stream->finish();
 }
 
 void svg_poly(int n, double *x, double *y, int filled, const pGEcontext gc,
               pDevDesc dd) {
 
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
-  fputs("<polyline points='", svgd->file);
+  SvgStreamPtr stream = svgd->stream;
+
+  (*stream) << "<polyline points='";
 
   for (int i = 0; i < n; i++) {
-    fprintf(svgd->file, "%.2f,%.2f ", x[i], y[i]);
+    (*stream) << x[i] << ',' << y[i] << ' ';
   }
-  fputs("'", svgd->file);
+  stream->put('\'');
 
-  write_style_begin(svgd->file);
-  write_style_linetype(svgd->file, gc, true);
+  write_style_begin(stream);
+  write_style_linetype(stream, gc, true);
   if (filled)
-    write_style_col(svgd->file, "fill", gc->fill);
-  write_style_end(svgd->file);
+    write_style_col(stream, "fill", gc->fill);
+  write_style_end(stream);
+
+  write_attr_clip(stream, svgd->clipno);
+
+  (*stream) << " />\n";
+  stream->flush();
 
-  fputs(" />\n", svgd->file);
 }
 
 void svg_polyline(int n, double *x, double *y, const pGEcontext gc,
@@ -342,33 +385,38 @@ void svg_path(double *x, double *y,
               Rboolean winding,
               const pGEcontext gc, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
+
   // Create path data
-  fputs("<path d='", svgd->file);
+  (*stream) << "<path d='";
   int ind = 0;
   for (int i = 0; i < npoly; i++) {
     // Move to the first point of the sub-path
-    fprintf(svgd->file, "M %.2f %.2f ", x[ind], y[ind]);
+    (*stream) << "M " << x[ind] << ' ' << y[ind] << ' ';
     ind++;
     // Draw the sub-path
     for (int j = 1; j < nper[i]; j++) {
-      fprintf(svgd->file, "L %.2f %.2f ", x[ind], y[ind]);
+      (*stream) << "L " << x[ind] << ' ' << y[ind] << ' ';
       ind++;
     }
     // Close the sub-path
-    fputs("Z ", svgd->file);
+    stream->put('Z');
   }
   // Finish path data
-  fputs("'", svgd->file);
+  stream->put('\'');
 
-  write_style_begin(svgd->file);
+  write_style_begin(stream);
   // Specify fill rule
-  write_style_str(svgd->file, "fill-rule", winding ? "nonzero" : "evenodd", true);
+  write_style_str(stream, "fill-rule", winding ? "nonzero" : "evenodd", true);
   if (is_filled(gc->fill))
-    write_style_col(svgd->file, "fill", gc->fill);
-  write_style_linetype(svgd->file, gc);
-  write_style_end(svgd->file);
+    write_style_col(stream, "fill", gc->fill);
+  write_style_linetype(stream, gc);
+  write_style_end(stream);
 
-  fputs(" />\n", svgd->file);
+  write_attr_clip(stream, svgd->clipno);
+
+  (*stream) << " />\n";
+  stream->flush();
 }
 
 double svg_strwidth(const char *str, const pGEcontext gc, pDevDesc dd) {
@@ -378,74 +426,96 @@ double svg_strwidth(const char *str, const pGEcontext gc, pDevDesc dd) {
     gc->cex * gc->ps, is_bold(gc->fontface), is_italic(gc->fontface));
   FontMetric fm = gdtools::context_extents(svgd->cc, std::string(str));
 
-  return fm.width;
+  return fm.width * 96 / 72;
 }
 
 void svg_rect(double x0, double y0, double x1, double y1,
               const pGEcontext gc, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
   // x and y give top-left position
-  fprintf(svgd->file,
-      "<rect x='%.2f' y='%.2f' width='%.2f' height='%.2f'",
-      fmin(x0, x1), fmin(y0, y1), fabs(x1 - x0), fabs(y1 - y0));
+  (*stream) << "<rect x='" << fmin(x0, x1) << "' y='" << fmin(y0, y1) <<
+    "' width='" << fabs(x1 - x0) << "' height='" << fabs(y1 - y0) << '\'';
 
-  write_style_begin(svgd->file);
-  write_style_linetype(svgd->file, gc, true);
+  write_style_begin(stream);
+  write_style_linetype(stream, gc, true);
   if (is_filled(gc->fill))
-    write_style_col(svgd->file, "fill", gc->fill);
-  write_style_end(svgd->file);
+    write_style_col(stream, "fill", gc->fill);
+  write_style_end(stream);
 
-  fputs(" />\n", svgd->file);
+  write_attr_clip(stream, svgd->clipno);
+
+  (*stream) << " />\n";
+  stream->flush();
 }
 
 void svg_circle(double x, double y, double r, const pGEcontext gc,
                        pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
-  fprintf(svgd->file, "<circle cx='%.2f' cy='%.2f' r='%.2fpt'", x, y, r);
+  (*stream) << "<circle cx='" << x << "' cy='" << y << "' r='" << r << "pt'";
 
-  write_style_begin(svgd->file);
-  write_style_linetype(svgd->file, gc, true);
+  write_style_begin(stream);
+  write_style_linetype(stream, gc, true);
   if (is_filled(gc->fill))
-    write_style_col(svgd->file, "fill", gc->fill);
-  write_style_end(svgd->file);
+    write_style_col(stream, "fill", gc->fill);
+  write_style_end(stream);
 
-  fputs(" />\n", svgd->file);
+  write_attr_clip(stream, svgd->clipno);
+
+  (*stream) << " />\n";
+  stream->flush();
 }
 
 void svg_text(double x, double y, const char *str, double rot,
               double hadj, const pGEcontext gc, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
-
-  fputs("<text", svgd->file);
-  if (rot == 0) {
-    write_attr_dbl(svgd->file, "x", x);
-    write_attr_dbl(svgd->file, "y", y);
-  } else {
-    fprintf(svgd->file, " transform='translate(%.2f,%.2f) rotate(%0.0f)'", x, y,
-      -1.0 * rot);
+  // If we specify the clip path inside <text>, the "transform" also
+  // affects the clip path, so we need to specify clip path at an outer level
+  if (svgd->clipno > 0) {
+    (*stream) << "<g";
+    write_attr_clip(stream, svgd->clipno);
+    stream->put('>');
   }
 
-  write_style_begin(svgd->file);
-  write_style_fontsize(svgd->file, gc->cex * gc->ps, true);
+  (*stream) << "<text";
+
+  if (rot == 0) {
+    write_attr_dbl(stream, "x", x);
+    write_attr_dbl(stream, "y", y);
+  } else {
+    (*stream) << tfm::format(" transform='translate(%0.2f,%0.2f) rotate(%0.0f)'",
+      x, y, -1.0 * rot);
+  }
+
+  write_style_begin(stream);
+  write_style_fontsize(stream, gc->cex * gc->ps, true);
   if (is_bold(gc->fontface))
-    write_style_str(svgd->file, "font-weight", "bold");
+    write_style_str(stream, "font-weight", "bold");
   if (is_italic(gc->fontface))
-    write_style_str(svgd->file, "font-style", "italic");
+    write_style_str(stream, "font-style", "italic");
   if (!is_black(gc->col))
-    write_style_col(svgd->file, "fill", gc->col);
+    write_style_col(stream, "fill", gc->col);
 
   std::string font = fontname(gc->fontfamily, gc->fontface);
-  write_style_str(svgd->file, "font-family", font.c_str());
-  write_style_end(svgd->file);
+  write_style_str(stream, "font-family", font.c_str());
+  write_style_end(stream);
 
-  fputs(">", svgd->file);
+  stream->put('>');
 
-  write_escaped(svgd->file, str);
+  write_escaped(stream, str);
 
-  fputs("</text>\n", svgd->file);
+  (*stream) << "</text>";
+
+  if (svgd->clipno > 0)
+    (*stream) << "</g>";
+
+  stream->put('\n');
+  stream->flush();
 }
 
 void svg_size(double *left, double *right, double *bottom, double *top,
@@ -463,6 +533,7 @@ void svg_raster(unsigned int *raster, int w, int h,
                 Rboolean interpolate,
                 const pGEcontext gc, pDevDesc dd) {
   SVGDesc *svgd = (SVGDesc*) dd->deviceSpecific;
+  SvgStreamPtr stream = svgd->stream;
 
   if (height < 0)
     height = -height;
@@ -475,23 +546,37 @@ void svg_raster(unsigned int *raster, int w, int h,
   std::string base64_str = gdtools::raster_to_str(raster_, w, h, width, height,
     (Rboolean) interpolate);
 
-  fputs("<image", svgd->file);
-  write_attr_dbl(svgd->file, "width", width);
-  write_attr_dbl(svgd->file, "height", height);
-  write_attr_dbl(svgd->file, "x", x);
-  write_attr_dbl(svgd->file, "y", y - height);
-
-  if( rot != 0 ){
-    fprintf(svgd->file, " transform='rotate(%0.0f,%.2f,%.2f)'", -1.0 * rot, x, y);
+  // If we specify the clip path inside <image>, the "transform" also
+  // affects the clip path, so we need to specify clip path at an outer level
+  if (svgd->clipno > 0) {
+    (*stream) << "<g";
+    write_attr_clip(stream, svgd->clipno);
+    stream->put('>');
   }
 
-  fprintf(svgd->file, " xlink:href='data:image/png;base64,%s'", base64_str.c_str());
-  fputs( "/>", svgd->file);
+  (*stream) << "<image";
+  write_attr_dbl(stream, "width", width);
+  write_attr_dbl(stream, "height", height);
+  write_attr_dbl(stream, "x", x);
+  write_attr_dbl(stream, "y", y - height);
+
+  if( rot != 0 ){
+    (*stream) << tfm::format(" transform='rotate(%0.0f,%.2f,%.2f)'", -1.0 * rot, x, y);
+  }
+
+  (*stream) << " xlink:href='data:image/png;base64," << base64_str << '\'';
+  (*stream) << "/>";
+
+  if (svgd->clipno > 0)
+    (*stream) << "</g>";
+
+  stream->put('\n');
+  stream->flush();
 }
 
 
-pDevDesc svg_driver_new(std::string filename, int bg, double width,
-                        double height, int pointsize, bool standalone) {
+pDevDesc svg_driver_new(SvgStreamPtr stream, int bg, double width,
+                        double height, double pointsize, bool standalone) {
 
   pDevDesc dd = (DevDesc*) calloc(1, sizeof(DevDesc));
   if (dd == NULL)
@@ -549,27 +634,26 @@ pDevDesc svg_driver_new(std::string filename, int bg, double width,
   dd->ipr[1] = 1.0 / 72.0;
 
   // Capabilities
-  dd->canClip = FALSE;
+  dd->canClip = TRUE;
   dd->canHAdj = 0;
   dd->canChangeGamma = FALSE;
   dd->displayListOn = FALSE;
   dd->haveTransparency = 2;
   dd->haveTransparentBg = 2;
 
-  dd->deviceSpecific = new SVGDesc(filename, standalone);
+  dd->deviceSpecific = new SVGDesc(stream, standalone);
   return dd;
 }
 
-// [[Rcpp::export]]
-bool devSVG_(std::string file, std::string bg_, int width, int height,
-             int pointsize, bool standalone) {
+void makeDevice(SvgStreamPtr stream, std::string bg_, double width, double height,
+  double pointsize, bool standalone) {
 
   int bg = R_GE_str2col(bg_.c_str());
 
   R_GE_checkVersionOrDie(R_GE_version);
   R_CheckDeviceAvailable();
   BEGIN_SUSPEND_INTERRUPTS {
-    pDevDesc dev = svg_driver_new(file, bg, width, height, pointsize, standalone);
+    pDevDesc dev = svg_driver_new(stream, bg, width, height, pointsize, standalone);
     if (dev == NULL)
       Rcpp::stop("Failed to start SVG device");
 
@@ -578,6 +662,24 @@ bool devSVG_(std::string file, std::string bg_, int width, int height,
     GEinitDisplayList(dd);
 
   } END_SUSPEND_INTERRUPTS;
+}
+
+// [[Rcpp::export]]
+bool svglite_(std::string file, std::string bg, double width, double height,
+             double pointsize, bool standalone) {
+
+  SvgStreamPtr stream(new SvgStreamFile(file));
+  makeDevice(stream, bg, width, height, pointsize, standalone);
+
+  return true;
+}
+
+// [[Rcpp::export]]
+bool svgstring_(Rcpp::Environment env, std::string bg, double width, double height,
+  double pointsize, bool standalone) {
+
+  SvgStreamPtr stream(new SvgStreamString(env));
+  makeDevice(stream, bg, width, height, pointsize, standalone);
 
   return true;
 }
